@@ -2,7 +2,7 @@ import functools
 from enum import Enum
 from typing import Callable, Dict, Optional, TypeVar, Union
 
-from marshmallow.fields import Field as MarshmallowField
+from marshmallow.fields import Field as MarshmallowField  # type: ignore
 
 from dataclasses_json.stringcase import (camelcase, pascalcase, snakecase,
                                          spinalcase)  # type: ignore
@@ -16,17 +16,20 @@ class Exclude:
     Pre-defined constants for exclusion. By default, fields are configured to
     be included.
     """
-    ALWAYS: Callable[[T], bool] = lambda _: True
-    NEVER: Callable[[T], bool] = lambda _: False
+    ALWAYS: Callable[[object], bool] = lambda _: True
+    NEVER: Callable[[object], bool] = lambda _: False
 
 
 # TODO: add warnings?
 class _GlobalConfig:
 
     def __init__(self):
-        self.encoders: Dict[type, Callable] = {}
-        self.decoders: Dict[type, Callable] = {}
-        self.mm_fields: Dict[type, MarshmallowField] = {}
+        self.encoders: Dict[Union[type, Optional[type]], Callable] = {}
+        self.decoders: Dict[Union[type, Optional[type]], Callable] = {}
+        self.mm_fields: Dict[
+            Union[type, Optional[type]],
+            MarshmallowField
+        ] = {}
         # self._json_module = json
 
     # TODO: #180
@@ -51,16 +54,16 @@ class LetterCase(Enum):
     PASCAL = pascalcase
 
 
-def config(metadata: dict = None, *,
+def config(metadata: Optional[dict] = None, *,
            # TODO: these can be typed more precisely
            # Specifically, a Callable[A, B], where `B` is bound as a JSON type
-           encoder: Callable = None,
-           decoder: Callable = None,
-           mm_field: MarshmallowField = None,
+           encoder: Optional[Callable] = None,
+           decoder: Optional[Callable] = None,
+           mm_field: Optional[MarshmallowField] = None,
            letter_case: Union[Callable[[str], str], LetterCase, None] = None,
            undefined: Optional[Union[str, Undefined]] = None,
-           field_name: str = None,
-           exclude: Union[Callable[[str, T], bool], Exclude, None] = None,
+           field_name: Optional[str] = None,
+           exclude: Optional[Callable[[T], bool]] = None,
            ) -> Dict[str, dict]:
     if metadata is None:
         metadata = {}
