@@ -3,6 +3,8 @@ import json
 import sys
 import warnings
 from collections import defaultdict, namedtuple
+from collections.abc import (Collection as ABCCollection, Mapping as ABCMapping, MutableMapping, MutableSequence,
+                             MutableSet, Sequence, Set)
 from dataclasses import (MISSING,
                          fields,
                          is_dataclass  # type: ignore
@@ -10,6 +12,7 @@ from dataclasses import (MISSING,
 from datetime import datetime, timezone
 from decimal import Decimal
 from enum import Enum
+from types import MappingProxyType
 from typing import (Any, Collection, Mapping, Union, get_type_hints,
                     Tuple, TypeVar, Type)
 from uuid import UUID
@@ -31,6 +34,15 @@ Json = Union[dict, list, str, int, float, bool, None]
 
 confs = ['encoder', 'decoder', 'mm_field', 'letter_case', 'exclude']
 FieldOverride = namedtuple('FieldOverride', confs)  # type: ignore
+collections_abc_type_to_implementation_type = MappingProxyType({
+    ABCCollection: tuple,
+    ABCMapping: dict,
+    MutableMapping: dict,
+    MutableSequence: list,
+    MutableSet: set,
+    Sequence: tuple,
+    Set: frozenset,
+})
 
 
 class _ExtendedEncoder(json.JSONEncoder):
@@ -231,10 +243,10 @@ def _decode_dataclass(cls, kvs, infer_missing):
 def _decode_type(type_, value, infer_missing):
     if _has_decoder_in_global_config(type_):
         return _get_decoder_in_global_config(type_)(value)
-    if is_dataclass(type_) or is_dataclass(type_):
-        return _decode_dataclass(type_, value, infer_missing)
     if _is_supported_generic(type_):
         return _decode_generic(type_, value, infer_missing)
+    if is_dataclass(type_) or is_dataclass(value):
+        return _decode_dataclass(type_, value, infer_missing)
     return _support_extended_types(type_, value)
 
 
@@ -302,14 +314,8 @@ def _decode_generic(type_, value, infer_missing):
         else:
             xs = _decode_items(_get_type_arg_param(type_, 0), value, infer_missing)
 
-        # get the constructor if using corresponding generic type in `typing`
-        # otherwise fallback on constructing using type_ itself
-        materialize_type = type_
-        try:
-            materialize_type = _get_type_cons(type_)
-        except (TypeError, AttributeError):
-            pass
-        res = materialize_type(xs)
+        collection_type = _resolve_collection_type_to_decode_to(type_)
+        res = collection_type(xs)
     elif _is_generic_dataclass(type_):
         origin = _get_type_origin(type_)
         res = _decode_dataclass(origin, value, infer_missing)
@@ -378,9 +384,11 @@ def _decode_items(type_args, xs, infer_missing):
     hence the check of `is_dataclass(vs)`
     """
     def handle_pep0673(pre_0673_hint: str) -> Union[Type, str]:
-        for module in sys.modules:
-            maybe_resolved = getattr(sys.modules[module], type_args, None)
-            if maybe_resolved:
+        for module in sys.modules.values():
+            if hasattr(module, type_args):
+                maybe_resolved = getattr(module, type_args)
+                warnings.warn(f"Assuming hint {pre_0673_hint} resolves to {maybe_resolved} "
+                              "This is not necessarily the value that is in-scope.")
                 return maybe_resolved
 
         warnings.warn(f"Could not resolve self-reference for type {pre_0673_hint}, "
@@ -402,6 +410,18 @@ def _decode_items(type_args, xs, infer_missing):
     return list(_decode_type(type_args, x, infer_missing) for x in xs)
 
 
+def _resolve_collection_type_to_decode_to(type_):
+    # get the constructor if using corresponding generic type in `typing`
+    # otherwise fallback on constructing using type_ itself
+    try:
+        collection_type = _get_type_cons(type_)
+    except (TypeError, AttributeError):
+        collection_type = type_
+
+    # map abstract collection to concrete implementation
+    return collections_abc_type_to_implementation_type.get(collection_type, collection_type)
+
+
 def _asdict(obj, encode_json=False):
     """
     A re-implementation of `asdict` (based on the original in the `dataclasses`
