@@ -1,5 +1,7 @@
 from dataclasses import dataclass, field
 
+import pytest
+
 from dataclasses_json import LetterCase, dataclass_json, config
 
 
@@ -56,6 +58,31 @@ class CamelCasePersonWithOverride:
     years_on_earth: int = field(metadata=config(field_name='age'))
 
 
+@dataclass_json
+@dataclass
+class CamelCaseProtectedNamePerson:
+    _given_name_2: str = field(
+        metadata={'dataclasses_json': {
+            'letter_case': LetterCase.CAMEL
+        }}
+    )
+
+
+@dataclass_json
+@dataclass
+class CamelCaseDuplicatedNameEncodingPerson:
+    given_name_1: str = field(
+        metadata={'dataclasses_json': {
+            'letter_case': LetterCase.CAMEL
+        }}
+    )
+    given_name1: str = field(
+        metadata={'dataclasses_json': {
+            'letter_case': LetterCase.CAMEL
+        }}
+    )
+
+
 class TestLetterCase:
     def test_camel_encode(self):
         assert CamelCasePerson('Alice').to_json() == '{"givenName": "Alice"}'
@@ -107,3 +134,13 @@ class TestLetterCase:
 
     def test_to_dict(self):
         assert {'givenName': 'Alice'} == CamelCasePerson('Alice').to_dict()
+
+    def test_protected_encode(self):
+        assert CamelCaseProtectedNamePerson('Alice').to_json() == '{"givenName2": "Alice"}'
+
+    def test_protected_decode(self):
+        assert CamelCaseProtectedNamePerson.from_json('{"givenName2": "Alice"}') == CamelCaseProtectedNamePerson('Alice')
+
+    def test_duplicated_encoding(self):
+        with pytest.raises(ValueError):
+            CamelCaseDuplicatedNameEncodingPerson('Alice', 'Bob').to_json()
